@@ -29,25 +29,43 @@ typedef enum LocaleVariable {
         _VARIABLE_LC_INVALID = -1
 } LocaleVariable;
 
+#if 0 /// UNNEEDED by elogind
 int get_locales(char ***l);
+#endif // 0
 bool locale_is_valid(const char *name);
 
 #define _(String) gettext(String)
 #define N_(String) String
+#if 0 /// UNNEEDED by elogind
 void init_gettext(void);
+#endif // 0
 
 bool is_locale_utf8(void);
 
 typedef enum {
-        TREE_VERTICAL,
-        TREE_BRANCH,
-        TREE_RIGHT,
-        TREE_SPACE,
-        TRIANGULAR_BULLET,
-        BLACK_CIRCLE,
-        ARROW,
-        MDASH,
-        ELLIPSIS,
+        SPECIAL_GLYPH_TREE_VERTICAL,
+        SPECIAL_GLYPH_TREE_BRANCH,
+        SPECIAL_GLYPH_TREE_RIGHT,
+        SPECIAL_GLYPH_TREE_SPACE,
+        SPECIAL_GLYPH_TRIANGULAR_BULLET,
+        SPECIAL_GLYPH_BLACK_CIRCLE,
+        SPECIAL_GLYPH_BULLET,
+        SPECIAL_GLYPH_MU,
+        SPECIAL_GLYPH_CHECK_MARK,
+        SPECIAL_GLYPH_CROSS_MARK,
+        SPECIAL_GLYPH_ARROW,
+        SPECIAL_GLYPH_ELLIPSIS,
+        SPECIAL_GLYPH_LIGHT_SHADE,
+        SPECIAL_GLYPH_DARK_SHADE,
+        SPECIAL_GLYPH_SIGMA,
+        _SPECIAL_GLYPH_FIRST_SMILEY,
+        SPECIAL_GLYPH_ECSTATIC_SMILEY = _SPECIAL_GLYPH_FIRST_SMILEY,
+        SPECIAL_GLYPH_HAPPY_SMILEY,
+        SPECIAL_GLYPH_SLIGHTLY_HAPPY_SMILEY,
+        SPECIAL_GLYPH_NEUTRAL_SMILEY,
+        SPECIAL_GLYPH_SLIGHTLY_UNHAPPY_SMILEY,
+        SPECIAL_GLYPH_UNHAPPY_SMILEY,
+        SPECIAL_GLYPH_DEPRESSED_SMILEY,
         _SPECIAL_GLYPH_MAX
 } SpecialGlyph;
 
@@ -56,12 +74,16 @@ const char *special_glyph(SpecialGlyph code) _const_;
 const char* locale_variable_to_string(LocaleVariable i) _const_;
 LocaleVariable locale_variable_from_string(const char *s) _pure_;
 
-int get_keymaps(char ***l);
-bool keymap_is_valid(const char *name);
-
 static inline void freelocalep(locale_t *p) {
         if (*p == (locale_t) 0)
                 return;
 
         freelocale(*p);
 }
+
+#if 0 /// UNNEEDED by elogind
+void locale_variables_free(char* l[_VARIABLE_LC_MAX]);
+static inline void locale_variables_freep(char*(*l)[_VARIABLE_LC_MAX]) {
+        locale_variables_free(*l);
+}
+#endif // 0
