@@ -5,29 +5,31 @@
 
 #include "log.h"
 
-enum {
-        PROC_CMDLINE_STRIP_RD_PREFIX = 1,
-        PROC_CMDLINE_VALUE_OPTIONAL = 2,
-};
+typedef enum ProcCmdlineFlags {
+        PROC_CMDLINE_STRIP_RD_PREFIX = 1 << 0,
+        PROC_CMDLINE_VALUE_OPTIONAL  = 1 << 1,
+        PROC_CMDLINE_RD_STRICT       = 1 << 2,
+} ProcCmdlineFlags;
 
 typedef int (*proc_cmdline_parse_t)(const char *key, const char *value, void *data);
 
 int proc_cmdline(char **ret);
 
-int proc_cmdline_parse_given(const char *line, proc_cmdline_parse_t parse_item, void *data, unsigned flags);
-int proc_cmdline_parse(const proc_cmdline_parse_t parse, void *userdata, unsigned flags);
+int proc_cmdline_parse_given(const char *line, proc_cmdline_parse_t parse_item, void *data, ProcCmdlineFlags flags);
+int proc_cmdline_parse(const proc_cmdline_parse_t parse, void *userdata, ProcCmdlineFlags flags);
 
-#if 0 /// UNNEEDED by elogind
-int proc_cmdline_get_key(const char *parameter, unsigned flags, char **value);
+int proc_cmdline_get_key(const char *parameter, ProcCmdlineFlags flags, char **value);
 int proc_cmdline_get_bool(const char *key, bool *ret);
 
-char *proc_cmdline_key_startswith(const char *s, const char *prefix);
+#if 0 /// UNNEEDED by elogind
+int proc_cmdline_get_key_many_internal(ProcCmdlineFlags flags, ...);
+#define proc_cmdline_get_key_many(flags, ...) proc_cmdline_get_key_many_internal(flags, __VA_ARGS__, NULL)
 #endif // 0
+
+char *proc_cmdline_key_startswith(const char *s, const char *prefix);
 bool proc_cmdline_key_streq(const char *x, const char *y);
 
 #if 0 /// UNNEEDED by elogind
-int shall_restore_state(void);
-const char* runlevel_to_target(const char *rl);
 #endif // 0
 
 /* A little helper call, to be used in proc_cmdline_parse_t callbacks */
