@@ -4,7 +4,6 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <string.h>
 
 #include "alloc-util.h"
 //#include "glob-util.h"
@@ -141,12 +140,10 @@ int unit_name_to_prefix(const char *n, char **ret) {
         return 0;
 }
 
-int unit_name_to_instance(const char *n, char **instance) {
+int unit_name_to_instance(const char *n, char **ret) {
         const char *p, *d;
-        char *i;
 
         assert(n);
-        assert(instance);
 
         if (!unit_name_is_valid(n, UNIT_NAME_ANY))
                 return -EINVAL;
@@ -154,8 +151,9 @@ int unit_name_to_instance(const char *n, char **instance) {
         /* Everything past the first @ and before the last . is the instance */
         p = strchr(n, '@');
         if (!p) {
-                *instance = NULL;
-                return 0;
+                if (ret)
+                        *ret = NULL;
+                return UNIT_NAME_PLAIN;
         }
 
         p++;
@@ -164,12 +162,14 @@ int unit_name_to_instance(const char *n, char **instance) {
         if (!d)
                 return -EINVAL;
 
-        i = strndup(p, d-p);
-        if (!i)
-                return -ENOMEM;
+        if (ret) {
+                char *i = strndup(p, d-p);
+                if (!i)
+                        return -ENOMEM;
 
-        *instance = i;
-        return 1;
+                *ret = i;
+        }
+        return d > p ? UNIT_NAME_INSTANCE : UNIT_NAME_TEMPLATE;
 }
 
 int unit_name_to_prefix_and_instance(const char *n, char **ret) {
@@ -405,7 +405,7 @@ int unit_name_path_escape(const char *f, char **ret) {
 }
 
 int unit_name_path_unescape(const char *f, char **ret) {
-        char *s;
+        _cleanup_free_ char *s = NULL;
         int r;
 
         assert(f);
@@ -418,34 +418,27 @@ int unit_name_path_unescape(const char *f, char **ret) {
                 if (!s)
                         return -ENOMEM;
         } else {
-                char *w;
+                _cleanup_free_ char *w = NULL;
 
                 r = unit_name_unescape(f, &w);
                 if (r < 0)
                         return r;
 
                 /* Don't accept trailing or leading slashes */
-                if (startswith(w, "/") || endswith(w, "/")) {
-                        free(w);
+                if (startswith(w, "/") || endswith(w, "/"))
                         return -EINVAL;
-                }
 
                 /* Prefix a slash again */
-                s = strappend("/", w);
-                free(w);
+                s = strjoin("/", w);
                 if (!s)
                         return -ENOMEM;
 
-                if (!path_is_normalized(s)) {
-                        free(s);
+                if (!path_is_normalized(s))
                         return -EINVAL;
-                }
         }
 
         if (ret)
-                *ret = s;
-        else
-                free(s);
+                *ret = TAKE_PTR(s);
 
         return 0;
 }
@@ -522,7 +515,7 @@ int unit_name_from_path(const char *path, const char *suffix, char **ret) {
         if (r < 0)
                 return r;
 
-        s = strappend(p, suffix);
+        s = strjoin(p, suffix);
         if (!s)
                 return -ENOMEM;
 
@@ -606,10 +599,10 @@ static bool do_escape_mangle(const char *f, bool allow_globs, char *t) {
  *
  *  If @allow_globs, globs characters are preserved. Otherwise, they are escaped.
  */
-int unit_name_mangle_with_suffix(const char *name, UnitNameMangle flags, const char *suffix, char **ret) {
+int unit_name_mangle_with_suffix(const char *name, const char *operation, UnitNameMangle flags, const char *suffix, char **ret) {
         char *s;
         int r;
-        bool mangled;
+        bool mangled, suggest_escape = true;
 
         assert(name);
         assert(suffix);
@@ -626,10 +619,14 @@ int unit_name_mangle_with_suffix(const char *name, UnitNameMangle flags, const c
                 goto good;
 
         /* Already a fully valid globbing expression? If so, no mangling is necessary either... */
-        if ((flags & UNIT_NAME_MANGLE_GLOB) &&
-            string_is_glob(name) &&
-            in_charset(name, VALID_CHARS_GLOB))
-                goto good;
+        if (string_is_glob(name) && in_charset(name, VALID_CHARS_GLOB)) {
+                if (flags & UNIT_NAME_MANGLE_GLOB)
+                        goto good;
+                log_full(flags & UNIT_NAME_MANGLE_WARN ? LOG_NOTICE : LOG_DEBUG,
+                         "Glob pattern passed%s%s, but globs are not supported for this.",
+                         operation ? " " : "", strempty(operation));
+                suggest_escape = false;
+        }
 
         if (is_device_path(name)) {
                 r = unit_name_from_path(name, ".device", ret);
@@ -654,11 +651,12 @@ int unit_name_mangle_with_suffix(const char *name, UnitNameMangle flags, const c
         mangled = do_escape_mangle(name, flags & UNIT_NAME_MANGLE_GLOB, s);
         if (mangled)
                 log_full(flags & UNIT_NAME_MANGLE_WARN ? LOG_NOTICE : LOG_DEBUG,
-                         "Invalid unit name \"%s\" was escaped as \"%s\" (maybe you should use systemd-escape?)",
-                         name, s);
+                         "Invalid unit name \"%s\" escaped as \"%s\"%s.",
+                         name, s,
+                         suggest_escape ? " (maybe you should use systemd-escape?)" : "");
 
-        /* Append a suffix if it doesn't have any, but only if this is not a glob, so that we can allow "foo.*" as a
-         * valid glob. */
+        /* Append a suffix if it doesn't have any, but only if this is not a glob, so that we can allow
+         * "foo.*" as a valid glob. */
         if ((!(flags & UNIT_NAME_MANGLE_GLOB) || !string_is_glob(s)) && unit_name_to_type(s) < 0)
                 strcat(s, suffix);
 
@@ -723,7 +721,7 @@ int slice_build_subslice(const char *slice, const char *name, char **ret) {
                 return -EINVAL;
 
         if (streq(slice, SPECIAL_ROOT_SLICE))
-                subslice = strappend(name, ".slice");
+                subslice = strjoin(name, ".slice");
         else {
                 char *e;
 
@@ -777,3 +775,5 @@ bool slice_name_is_valid(const char *name) {
 
         return true;
 }
+#if 0 /// UNNEEDED by elogind
+#endif // 0
