@@ -10,16 +10,17 @@
 #include "alloc-util.h"
 #include "bus-util.h"
 #include "cgroup-util.h"
+#include "conf-files.h"
+#include "cgroup-setup.h"
 #include "dev-setup.h"
 #include "dirent-util.h"
-#include "efivars.h"
+#include "efi-loader.h"
 #include "fd-util.h"
 #include "fileio.h"
 #include "fs-util.h"
 #include "label.h"
 #include "log.h"
 #include "macro.h"
-#include "missing.h"
 #include "mkdir.h"
 #include "mount-setup.h"
 #include "mountpoint-util.h"
@@ -113,7 +114,7 @@ static const MountPoint mount_table[] = {
 #if 0 /// UNNEEDED by elogind
         { "bpf",         "/sys/fs/bpf",               "bpf",        "mode=700",                MS_NOSUID|MS_NOEXEC|MS_NODEV,
           NULL,          MNT_NONE,                  },
-#else
+#else // 0
         { "cgroup",      "/sys/fs/cgroup/elogind",    "cgroup",     "none,name=elogind,release_agent="SYSTEMD_CGROUP_AGENT_PATH",xattr", MS_NOSUID|MS_NOEXEC|MS_NODEV,
           cg_is_legacy_wanted, MNT_IN_CONTAINER  },
         { "cgroup",      "/sys/fs/cgroup/elogind",    "cgroup",     "none,name=elogind,release_agent="SYSTEMD_CGROUP_AGENT_PATH,         MS_NOSUID|MS_NOEXEC|MS_NODEV,
@@ -425,7 +426,8 @@ static int relabel_cgroup_filesystems(void) {
 }
 
 static int relabel_extra(void) {
-        _cleanup_closedir_ DIR *d = NULL;
+        _cleanup_strv_free_ char **files = NULL;
+        char **file;
         int r, c = 0;
 
         /* Support for relabelling additional files or directories after loading the policy. For this, code in the
@@ -436,55 +438,27 @@ static int relabel_extra(void) {
          * possible.
          */
 
-        d = opendir("/run/systemd/relabel-extra.d/");
-        if (!d) {
-                if (errno == ENOENT)
-                        return 0;
+        r = conf_files_list(&files, ".relabel", NULL,
+                            CONF_FILES_FILTER_MASKED | CONF_FILES_REGULAR,
+                            "/run/systemd/relabel-extra.d/");
+        if (r < 0)
+                return log_error_errno(r, "Failed to enumerate /run/systemd/relabel-extra.d/, ignoring: %m");
 
-                return log_warning_errno(errno, "Failed to open /run/systemd/relabel-extra.d/, ignoring: %m");
-        }
-
-        for (;;) {
+        STRV_FOREACH(file, files) {
                 _cleanup_fclose_ FILE *f = NULL;
-                _cleanup_close_ int fd = -1;
-                struct dirent *de;
 
-                errno = 0;
-                de = readdir_no_dot(d);
-                if (!de) {
-                        if (errno != 0)
-                                return log_error_errno(errno, "Failed read directory /run/systemd/relabel-extra.d/, ignoring: %m");
-                        break;
-                }
-
-                if (hidden_or_backup_file(de->d_name))
-                        continue;
-
-                if (!endswith(de->d_name, ".relabel"))
-                        continue;
-
-                if (!IN_SET(de->d_type, DT_REG, DT_UNKNOWN))
-                        continue;
-
-                fd = openat(dirfd(d), de->d_name, O_RDONLY|O_CLOEXEC|O_NONBLOCK);
-                if (fd < 0) {
-                        log_warning_errno(errno, "Failed to open /run/systemd/relabel-extra.d/%s, ignoring: %m", de->d_name);
-                        continue;
-                }
-
-                f = fdopen(fd, "r");
+                f = fopen(*file, "re");
                 if (!f) {
-                        log_warning_errno(errno, "Failed to convert file descriptor into file object, ignoring: %m");
+                        log_warning_errno(errno, "Failed to open %s, ignoring: %m", *file);
                         continue;
                 }
-                TAKE_FD(fd);
 
                 for (;;) {
                         _cleanup_free_ char *line = NULL;
 
                         r = read_line(f, LONG_LINE_MAX, &line);
                         if (r < 0) {
-                                log_warning_errno(r, "Failed to read from /run/systemd/relabel-extra.d/%s, ignoring: %m", de->d_name);
+                                log_warning_errno(r, "Failed to read %s, ignoring: %m", *file);
                                 break;
                         }
                         if (r == 0) /* EOF */
@@ -508,8 +482,8 @@ static int relabel_extra(void) {
                         c++;
                 }
 
-                if (unlinkat(dirfd(d), de->d_name, 0) < 0)
-                        log_warning_errno(errno, "Failed to remove /run/systemd/relabel-extra.d/%s, ignoring: %m", de->d_name);
+                if (unlink(*file) < 0)
+                        log_warning_errno(errno, "Failed to remove %s, ignoring: %m", *file);
         }
 
         /* Remove when we complete things. */
@@ -584,7 +558,7 @@ int mount_setup(bool loaded_policy) {
 
         /* Also create /run/systemd/inaccessible nodes, so that we always have something to mount inaccessible nodes
          * from. */
-        (void) make_inaccessible_nodes(NULL, UID_INVALID, GID_INVALID);
+        (void) make_inaccessible_nodes("/run/systemd", UID_INVALID, GID_INVALID);
 
         return 0;
 }
