@@ -1,5 +1,8 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
+#include <unistd.h>
+#include <sys/types.h>
+
 #include "bus-internal.h"
 #include "bus-message.h"
 #include "bus-signature.h"
