@@ -2,7 +2,6 @@
 #pragma once
 
 #include <pthread.h>
-#include <sys/socket.h>
 
 #include "sd-bus.h"
 
@@ -18,7 +17,16 @@
 
 /* Note that we use the new /run prefix here (instead of /var/run) since we require them to be aliases and
  * that way we become independent of /var being mounted */
+#if 0 /// elogind supports both /var/run and /run
 #define DEFAULT_SYSTEM_BUS_ADDRESS "unix:path=/run/dbus/system_bus_socket"
+#else // 0
+/* Not all systems have dbus hierarchy in /run (as preferred by systemd) */
+#if VARRUN_IS_SYMLINK
+  #define DEFAULT_SYSTEM_BUS_ADDRESS "unix:path=/run/dbus/system_bus_socket"
+#else // 0
+  #define DEFAULT_SYSTEM_BUS_ADDRESS "unix:path=/var/run/dbus/system_bus_socket"
+#endif // VARRUN_IS_SYMLINK
+#endif // 0
 #define DEFAULT_USER_BUS_ADDRESS_FMT "unix:path=%s/bus"
 
 struct reply_callback {
