@@ -8,8 +8,6 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
 #include <unistd.h>
@@ -437,7 +435,7 @@ _public_ int sd_is_mq(int fd, const char *path) {
         }
 
         return 1;
-#else
+#else // 0
         return -ENOSYS;
 #endif // 0
 }
@@ -616,7 +614,7 @@ _public_ int sd_booted(void) {
                 return false;
 
         return -errno;
-#else
+#else // 0
         return 0;
 #endif // 0
 }
