@@ -3,7 +3,7 @@
 #include <errno.h>
 #include <linux/filter.h>
 #include <linux/netlink.h>
-#include <sys/socket.h>
+#include <unistd.h>
 
 #include "sd-device.h"
 #include "sd-event.h"
@@ -17,7 +17,6 @@
 #include "format-util.h"
 #include "hashmap.h"
 #include "io-util.h"
-#include "missing.h"
 #include "mountpoint-util.h"
 #include "set.h"
 #include "socket-util.h"
@@ -109,11 +108,13 @@ int device_monitor_disconnect(sd_device_monitor *m) {
         return 0;
 }
 
+#if 0 /// UNNEEDED by elogind
 int device_monitor_get_fd(sd_device_monitor *m) {
         assert_return(m, -EINVAL);
 
         return m->sock;
 }
+#endif // 0
 
 int device_monitor_new_full(sd_device_monitor **ret, MonitorNetlinkGroup group, int fd) {
         _cleanup_(sd_device_monitor_unrefp) sd_device_monitor *m = NULL;
