@@ -2,7 +2,6 @@
 
 #include <errno.h>
 #include <poll.h>
-#include <string.h>
 #include <sys/inotify.h>
 #include <unistd.h>
 
@@ -59,7 +58,7 @@ _public_ int sd_pid_get_unit(pid_t pid, char **unit) {
 #if 0 /// elogind does not support systemd units
         r = cg_pid_get_unit(pid, unit);
         return IN_SET(r, -ENXIO, -ENOMEDIUM) ? -ENODATA : r;
-#else
+#else // 0
         return -ESRCH;
 #endif // 0
 }
@@ -75,7 +74,7 @@ _public_ int sd_pid_get_user_unit(pid_t pid, char **unit) {
 #if 0 /// elogind does not support systemd units
         r = cg_pid_get_user_unit(pid, unit);
         return IN_SET(r, -ENXIO, -ENOMEDIUM) ? -ENODATA : r;
-#else
+#else // 0
         return -ESRCH;
 #endif // 0
 }
@@ -91,7 +90,7 @@ _public_ int sd_pid_get_machine_name(pid_t pid, char **name) {
 #if 0 /// elogind does not support systemd units
         r = cg_pid_get_machine_name(pid, name);
         return IN_SET(r, -ENXIO, -ENOMEDIUM) ? -ENODATA : r;
-#else
+#else // 0
         return -ESRCH;
 #endif // 0
 }
@@ -193,7 +192,7 @@ _public_ int sd_peer_get_unit(int fd, char **unit) {
 
 #if 0 /// elogind does not support systemd units
         return cg_pid_get_unit(ucred.pid, unit);
-#else
+#else // 0
         return -ESRCH;
 #endif // 0
 }
@@ -211,7 +210,7 @@ _public_ int sd_peer_get_user_unit(int fd, char **unit) {
 
 #if 0 /// elogind does not support systemd units
         return cg_pid_get_user_unit(ucred.pid, unit);
-#else
+#else // 0
         return -ESRCH;
 #endif // 0
 }
@@ -229,7 +228,7 @@ _public_ int sd_peer_get_machine_name(int fd, char **machine) {
 
 #if 0 /// elogind does not support systemd units
         return cg_pid_get_machine_name(ucred.pid, machine);
-#else
+#else // 0
         return -ESRCH;
 #endif // 0
 }
@@ -959,7 +958,8 @@ _public_ int sd_machine_get_ifindices(const char *machine, int **ifindices) {
 
                 *(char*) (mempcpy(buf, word, l)) = 0;
 
-                if (parse_ifindex(buf, &ifi) < 0)
+                ifi = parse_ifindex(buf);
+                if (ifi < 0)
                         continue;
 
                 if (!GREEDY_REALLOC(ni, allocated, nr+1)) {
