@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
 #include <fcntl.h>
-#include <pwd.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <linux/vt.h>
@@ -28,6 +27,7 @@
 #include "terminal-util.h"
 #include "udev-util.h"
 #include "user-util.h"
+#include "userdb.h"
 /// Additional includes needed by elogind
 #include "elogind.h"
 
@@ -43,7 +43,7 @@ void manager_reset_config(Manager *m) {
 #if 0 /// elogind does not start a user service manager, the delay is unneeded.
         m->user_stop_delay = 10 * USEC_PER_SEC;
 
-#else
+#else // 0
         m->user_stop_delay = 0;
 #endif // 0
         m->handle_power_key = HANDLE_POWEROFF;
@@ -63,7 +63,6 @@ void manager_reset_config(Manager *m) {
         m->idle_action = HANDLE_IGNORE;
 
         m->runtime_dir_size = physical_memory_scale(10U, 100U); /* 10% */
-        m->user_tasks_max = system_tasks_max_scale(DEFAULT_USER_TASKS_MAX_PERCENTAGE, 100U); /* 33% */
         m->sessions_max = 8192;
         m->inhibitors_max = 8192;
 
@@ -133,7 +132,7 @@ int manager_parse_config_file(Manager *m) {
                                         CONF_PATHS_NULSTR("systemd/logind.conf.d"),
                                         "Login\0",
                                         config_item_perf_lookup, logind_gperf_lookup,
-#else
+#else // 0
         const char* logind_conf = getenv("ELOGIND_CONF_FILE");
 
         if (!logind_conf)
@@ -145,7 +144,7 @@ int manager_parse_config_file(Manager *m) {
                                         CONFIG_PARSE_WARN, m);
 }
 
-int manager_add_device(Manager *m, const char *sysfs, bool master, Device **_device) {
+int manager_add_device(Manager *m, const char *sysfs, bool master, Device **ret_device) {
         Device *d;
 
         assert(m);
@@ -161,13 +160,13 @@ int manager_add_device(Manager *m, const char *sysfs, bool master, Device **_dev
                         return -ENOMEM;
         }
 
-        if (_device)
-                *_device = d;
+        if (ret_device)
+                *ret_device = d;
 
         return 0;
 }
 
-int manager_add_seat(Manager *m, const char *id, Seat **_seat) {
+int manager_add_seat(Manager *m, const char *id, Seat **ret_seat) {
         Seat *s;
         int r;
 
@@ -181,13 +180,13 @@ int manager_add_seat(Manager *m, const char *id, Seat **_seat) {
                         return r;
         }
 
-        if (_seat)
-                *_seat = s;
+        if (ret_seat)
+                *ret_seat = s;
 
         return 0;
 }
 
-int manager_add_session(Manager *m, const char *id, Session **_session) {
+int manager_add_session(Manager *m, const char *id, Session **ret_session) {
         Session *s;
         int r;
 
@@ -201,35 +200,32 @@ int manager_add_session(Manager *m, const char *id, Session **_session) {
                         return r;
         }
 
-        if (_session)
-                *_session = s;
+        if (ret_session)
+                *ret_session = s;
 
         return 0;
 }
 
 int manager_add_user(
                 Manager *m,
-                uid_t uid,
-                gid_t gid,
-                const char *name,
-                const char *home,
-                User **_user) {
+                UserRecord *ur,
+                User **ret_user) {
 
         User *u;
         int r;
 
         assert(m);
-        assert(name);
+        assert(ur);
 
-        u = hashmap_get(m->users, UID_TO_PTR(uid));
+        u = hashmap_get(m->users, UID_TO_PTR(ur->uid));
         if (!u) {
-                r = user_new(&u, m, uid, gid, name, home);
+                r = user_new(&u, m, ur);
                 if (r < 0)
                         return r;
         }
 
-        if (_user)
-                *_user = u;
+        if (ret_user)
+                *ret_user = u;
 
         return 0;
 }
@@ -237,34 +233,37 @@ int manager_add_user(
 int manager_add_user_by_name(
                 Manager *m,
                 const char *name,
-                User **_user) {
+                User **ret_user) {
 
-        const char *home = NULL;
-        uid_t uid;
-        gid_t gid;
+        _cleanup_(user_record_unrefp) UserRecord *ur = NULL;
         int r;
 
         assert(m);
         assert(name);
 
-        r = get_user_creds(&name, &uid, &gid, &home, NULL, 0);
+        r = userdb_by_name(name, 0, &ur);
         if (r < 0)
                 return r;
 
-        return manager_add_user(m, uid, gid, name, home, _user);
+        return manager_add_user(m, ur, ret_user);
 }
 
-int manager_add_user_by_uid(Manager *m, uid_t uid, User **_user) {
-        struct passwd *p;
+int manager_add_user_by_uid(
+                Manager *m,
+                uid_t uid,
+                User **ret_user) {
+
+        _cleanup_(user_record_unrefp) UserRecord *ur = NULL;
+        int r;
 
         assert(m);
+        assert(uid_is_valid(uid));
 
-        errno = 0;
-        p = getpwuid(uid);
-        if (!p)
-                return errno_or_else(ENOENT);
+        r = userdb_by_uid(uid, 0, &ur);
+        if (r < 0)
+                return r;
 
-        return manager_add_user(m, uid, p->pw_gid, p->pw_name, p->pw_dir, _user);
+        return manager_add_user(m, ur, ret_user);
 }
 
 int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **ret) {
@@ -287,7 +286,7 @@ int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **ret) {
         return 0;
 }
 
-int manager_add_button(Manager *m, const char *name, Button **_button) {
+int manager_add_button(Manager *m, const char *name, Button **ret_button) {
         Button *b;
 
         assert(m);
@@ -300,8 +299,8 @@ int manager_add_button(Manager *m, const char *name, Button **_button) {
                         return -ENOMEM;
         }
 
-        if (_button)
-                *_button = b;
+        if (ret_button)
+                *ret_button = b;
 
         return 0;
 }
@@ -414,7 +413,7 @@ int manager_process_button_device(Manager *m, sd_device *d) {
 int manager_get_session_by_pid(Manager *m, pid_t pid, Session **ret) {
 #if 0 /// elogind does not support systemd units, but its own session system
         _cleanup_free_ char *unit = NULL;
-#else
+#else // 0
         _cleanup_free_ char *session_name = NULL;
 #endif // 0
         Session *s;
@@ -429,44 +428,33 @@ int manager_get_session_by_pid(Manager *m, pid_t pid, Session **ret) {
         if (!s) {
 #if 0 /// elogind does not support systemd units, but its own session system
                 r = cg_pid_get_unit(pid, &unit);
-                if (r < 0)
-                        goto not_found;
-
-                s = hashmap_get(m->session_units, unit);
-                if (!s)
-                        goto not_found;
-#else
+                if (r >= 0)
+                        s = hashmap_get(m->session_units, unit);
+#else // 0
                 log_debug_elogind("Searching session for PID %u", pid);
                 r = cg_pid_get_session(pid, &session_name);
-                if (r < 0)
-                        goto not_found;
 
-                s = hashmap_get(m->sessions, session_name);
-                if (!s)
-                        goto not_found;
+                if (r >= 0)
+                        s = hashmap_get(m->sessions, session_name);
+
                 log_debug_elogind("Session Name \"%s\" -> Session \"%s\"",
-                                  session_name, s && s->id ? s->id : "NULL");
+                                  strnull(session_name), s && s->id ? s->id : "(null)");
 #endif // 0
         }
 
         if (ret)
                 *ret = s;
 
-        return 1;
-
-not_found:
-        if (ret)
-                *ret = NULL;
-        return 0;
+        return !!s;
 }
 
 int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
 #if 0 /// elogind does not support systemd units, but its own session system
         _cleanup_free_ char *unit = NULL;
-        User *u;
-#else
+#else // 0
         Session *s;
 #endif // 0
+        User *u = NULL;
         int r;
 
         assert(m);
@@ -476,32 +464,19 @@ int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
 
 #if 0 /// elogind does not support systemd units, but its own session system
         r = cg_pid_get_slice(pid, &unit);
-        if (r < 0)
-                goto not_found;
-
-        u = hashmap_get(m->user_units, unit);
-        if (!u)
-                goto not_found;
+        if (r >= 0)
+                u = hashmap_get(m->user_units, unit);
+#else // 0
+        // If a session was found, ignore it if it is already closing.
+        r = manager_get_session_by_pid (m, pid, &s);
+        if ( (r >= 0) && (SESSION_CLOSING != session_get_state(s)) )
+                u = s->user;
+#endif // 0
 
         if (ret)
                 *ret = u;
 
-#else
-        // If a session was found, ignore it if it is already closing.
-        r = manager_get_session_by_pid (m, pid, &s);
-        if (r <= 0 || SESSION_CLOSING == session_get_state(s))
-                goto not_found;
-
-        if (ret)
-                *ret =  s->user;
-#endif // 0
-        return 1;
-
-not_found:
-        if (ret)
-                *ret = NULL;
-
-        return 0;
+        return !!u;
 }
 
 int manager_get_idle_hint(Manager *m, dual_timestamp *t) {
