@@ -3,6 +3,8 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include "alloc-util.h"
@@ -11,64 +13,80 @@
 #include "fd-util.h"
 #include "fileio.h"
 #include "format-util.h"
+#include "io-util.h"
+#include "logind-dbus.h"
 #include "logind-inhibit.h"
 #include "mkdir.h"
 #include "parse-util.h"
+#include "path-util.h"
 #include "string-table.h"
 #include "string-util.h"
 #include "tmpfile-util.h"
 #include "user-util.h"
 #include "util.h"
 
-Inhibitor* inhibitor_new(Manager *m, const char* id) {
-        Inhibitor *i;
+static void inhibitor_remove_fifo(Inhibitor *i);
 
+int inhibitor_new(Inhibitor **ret, Manager *m, const char* id) {
+        _cleanup_(inhibitor_freep) Inhibitor *i = NULL;
+        int r;
+
+        assert(ret);
         assert(m);
+        assert(id);
 
-        i = new0(Inhibitor, 1);
+        i = new(Inhibitor, 1);
         if (!i)
-                return NULL;
+                return -ENOMEM;
 
-        i->state_file = strappend("/run/systemd/inhibit/", id);
+        *i = (Inhibitor) {
+                .manager = m,
+                .what = _INHIBIT_WHAT_INVALID,
+                .mode = _INHIBIT_MODE_INVALID,
+                .uid = UID_INVALID,
+                .fifo_fd = -1,
+        };
+
+        i->state_file = path_join("/run/systemd/inhibit", id);
         if (!i->state_file)
-                return mfree(i);
+                return -ENOMEM;
 
         i->id = basename(i->state_file);
 
-        if (hashmap_put(m->inhibitors, i->id, i) < 0) {
-                free(i->state_file);
-                return mfree(i);
-        }
+        r = hashmap_put(m->inhibitors, i->id, i);
+        if (r < 0)
+                return r;
 
-        i->manager = m;
-        i->fifo_fd = -1;
-
-        return i;
+        *ret = TAKE_PTR(i);
+        return 0;
 }
 
-void inhibitor_free(Inhibitor *i) {
-        assert(i);
+Inhibitor* inhibitor_free(Inhibitor *i) {
 
         log_debug_elogind("Freeing inhibitor %s", i->id);
-        hashmap_remove(i->manager->inhibitors, i->id);
-
-        inhibitor_remove_fifo(i);
+        if (!i)
+                return NULL;
 
         free(i->who);
         free(i->why);
 
-        if (i->state_file) {
 #if 1 /// Do not remove the state file if elogind got interrupted
                 if (!i->manager->do_interrupt)
 #endif // 1
-                unlink(i->state_file);
-                free(i->state_file);
-        }
+        sd_event_source_unref(i->event_source);
+        safe_close(i->fifo_fd);
 
-        free(i);
+        hashmap_remove(i->manager->inhibitors, i->id);
+
+        /* Note that we don't remove neither the state file nor the fifo path here, since we want both to
+         * survive daemon restarts */
+        free(i->state_file);
+        free(i->fifo_path);
+
+        return mfree(i);
 }
 
-int inhibitor_save(Inhibitor *i) {
+static int inhibitor_save(Inhibitor *i) {
         _cleanup_free_ char *temp_path = NULL;
         _cleanup_fclose_ FILE *f = NULL;
         int r;
@@ -83,7 +101,7 @@ int inhibitor_save(Inhibitor *i) {
         if (r < 0)
                 goto fail;
 
-        fchmod(fileno(f), 0644);
+        (void) fchmod(fileno(f), 0644);
 
         fprintf(f,
                 "# This is private data. Do not parse.\n"
@@ -143,6 +161,16 @@ fail:
         return log_error_errno(r, "Failed to save inhibit data %s: %m", i->state_file);
 }
 
+static int bus_manager_send_inhibited_change(Inhibitor *i) {
+        const char *property;
+
+        assert(i);
+
+        property = i->mode == INHIBIT_BLOCK ? "BlockInhibited" : "DelayInhibited";
+
+        return manager_send_changed(i->manager, property, NULL);
+}
+
 int inhibitor_start(Inhibitor *i) {
         assert(i);
 
@@ -156,16 +184,16 @@ int inhibitor_start(Inhibitor *i) {
                   i->pid, i->uid,
                   inhibit_mode_to_string(i->mode));
 
-        inhibitor_save(i);
-
         i->started = true;
 
-        manager_send_changed(i->manager, i->mode == INHIBIT_BLOCK ? "BlockInhibited" : "DelayInhibited", NULL);
+        inhibitor_save(i);
+
+        bus_manager_send_inhibited_change(i);
 
         return 0;
 }
 
-int inhibitor_stop(Inhibitor *i) {
+void inhibitor_stop(Inhibitor *i) {
         assert(i);
 
         log_debug_elogind("Stopping inhibitor %s ...", i->id);
@@ -175,14 +203,14 @@ int inhibitor_stop(Inhibitor *i) {
                           i->pid, i->uid,
                           inhibit_mode_to_string(i->mode));
 
+        inhibitor_remove_fifo(i);
+
         if (i->state_file)
-                unlink(i->state_file);
+                (void) unlink(i->state_file);
 
         i->started = false;
 
-        manager_send_changed(i->manager, i->mode == INHIBIT_BLOCK ? "BlockInhibited" : "DelayInhibited", NULL);
-
-        return 0;
+        bus_manager_send_inhibited_change(i);
 }
 
 int inhibitor_load(Inhibitor *i) {
@@ -209,7 +237,7 @@ int inhibitor_load(Inhibitor *i) {
                            "MODE", &mode,
                            "FIFO", &i->fifo_path);
         if (r < 0)
-                return r;
+                return log_error_errno(r, "Failed to read %s: %m", i->state_file);
 
         w = what ? inhibit_what_from_string(what) : 0;
         if (w >= 0)
@@ -222,35 +250,33 @@ int inhibitor_load(Inhibitor *i) {
         if (uid) {
                 r = parse_uid(uid, &i->uid);
                 if (r < 0)
-                        return r;
+                        log_debug_errno(r, "Failed to parse UID of inhibitor: %s", uid);
         }
 
         if (pid) {
                 r = parse_pid(pid, &i->pid);
                 if (r < 0)
-                        return r;
+                        log_debug_errno(r, "Failed to parse PID of inhibitor: %s", pid);
         }
 
         if (who) {
                 r = cunescape(who, 0, &cc);
                 if (r < 0)
-                        return r;
+                        return log_oom();
 
-                free(i->who);
-                i->who = cc;
+                free_and_replace(i->who, cc);
         }
 
         if (why) {
                 r = cunescape(why, 0, &cc);
                 if (r < 0)
-                        return r;
+                        return log_oom();
 
-                free(i->why);
-                i->why = cc;
+                free_and_replace(i->why, cc);
         }
 
         if (i->fifo_path) {
-                int fd;
+                _cleanup_close_ int fd = -1;
 
 #if 1 /// elogind has no systemd in the background preserving FIFOs ...
 #if ENABLE_DEBUG_ELOGIND
@@ -271,8 +297,10 @@ int inhibitor_load(Inhibitor *i) {
                         i->fifo_path = mfree(i->fifo_path);
                 }
 #endif // 1
+                /* Let's re-open the FIFO on both sides, and close the writing side right away */
                 fd = inhibitor_create_fifo(i);
-                safe_close(fd);
+                if (fd < 0)
+                        return log_error_errno(fd, "Failed to reopen FIFO: %m");
         }
 
         return 0;
@@ -330,6 +358,8 @@ int inhibitor_create_fifo(Inhibitor *i) {
                 r = sd_event_source_set_priority(i->event_source, SD_EVENT_PRIORITY_IDLE-10);
                 if (r < 0)
                         return r;
+
+                (void) sd_event_source_set_description(i->event_source, "inhibitor-ref");
         }
 
         /* Open writing side */
@@ -341,11 +371,12 @@ int inhibitor_create_fifo(Inhibitor *i) {
         return r;
 }
 
-void inhibitor_remove_fifo(Inhibitor *i) {
+static void inhibitor_remove_fifo(Inhibitor *i) {
+        assert(i);
+
 #if 1 /// Don't keep invalid FIFOs on elogind restart
         int current_fifo_fd = i->fifo_fd;
 #endif // 1
-        assert(i);
 
         i->event_source = sd_event_source_unref(i->event_source);
         i->fifo_fd = safe_close(i->fifo_fd);
@@ -359,14 +390,32 @@ void inhibitor_remove_fifo(Inhibitor *i) {
                         log_debug_elogind("Removing FIFO %d at %s for inhibitor %s",
                                           current_fifo_fd, i->fifo_path, i->id);
 #endif // 1
-                unlink(i->fifo_path);
+                (void) unlink(i->fifo_path);
+                i->fifo_path = mfree(i->fifo_path);
 #if 1 /// Close elogind extra if
                 }
 #endif // 1
-                i->fifo_path = mfree(i->fifo_path);
         }
 }
 
+bool inhibitor_is_orphan(Inhibitor *i) {
+        assert(i);
+
+        if (!i->started)
+                return true;
+
+        if (!i->fifo_path)
+                return true;
+
+        if (i->fifo_fd < 0)
+                return true;
+
+        if (pipe_eof(i->fifo_fd) != 0)
+                return true;
+
+        return false;
+}
+
 InhibitWhat manager_inhibit_what(Manager *m, InhibitMode mm) {
         Inhibitor *i;
         Iterator j;
