@@ -5,7 +5,6 @@
 #include <linux/kd.h>
 #include <linux/vt.h>
 #include <signal.h>
-#include <string.h>
 #include <sys/ioctl.h>
 //#include <sys/stat.h>
 #include <unistd.h>
@@ -38,8 +37,8 @@
 #include "tmpfile-util.h"
 #include "user-util.h"
 #include "util.h"
-
 /// Additional includes needed by elogind
+#include "cgroup-setup.h"
 #include "extract-word.h"
 
 #define RELEASE_USEC (20*USEC_PER_SEC)
@@ -239,8 +238,8 @@ int session_save(Session *s) {
                 "IS_DISPLAY=%i\n"
                 "STATE=%s\n"
                 "REMOTE=%i\n",
-                s->user->uid,
-                s->user->name,
+                s->user->user_record->uid,
+                s->user->user_record->user_name,
                 session_is_active(s),
                 s->user->display == s,
                 session_state_to_string(session_get_state(s)),
@@ -479,7 +478,7 @@ int session_load(Session *s) {
                                                "User of session %s not known.",
                                                s->id);
 
-                log_debug_elogind("Attaching session %s to user %d", s->id, user->uid);
+                log_debug_elogind("Attaching session %s to user %d", s->id, user->user_record->uid);
                 session_set_user(s, user);
         }
 
@@ -675,7 +674,7 @@ static int session_start_scope(Session *s, sd_bus_message *properties, sd_bus_er
                 if (!scope)
                         return log_oom();
 
-                description = strjoina("Session ", s->id, " of user ", s->user->name);
+                description = strjoina("Session ", s->id, " of user ", s->user->user_record->user_name);
 
                 r = manager_start_scope(
                                 s->manager,
@@ -691,7 +690,7 @@ static int session_start_scope(Session *s, sd_bus_message *properties, sd_bus_er
                                           "systemd-user-sessions.service",
                                           s->user->runtime_dir_service,
                                           s->user->service),
-                                s->user->home,
+                                user_record_home_directory(s->user->user_record),
                                 properties,
                                 error,
                                 &s->scope_job);
@@ -706,7 +705,7 @@ static int session_start_scope(Session *s, sd_bus_message *properties, sd_bus_er
 
         return 0;
 }
-#else
+#else // 0
 static int session_start_cgroup(Session *s) {
         int r;
 
@@ -747,7 +746,7 @@ int session_start(Session *s, sd_bus_message *properties, sd_bus_error *error) {
 
 #if 0 /// elogind does its own session management
         r = session_start_scope(s, properties, error);
-#else
+#else // 0
         r = session_start_cgroup(s);
 #endif // 0
         if (r < 0)
@@ -756,9 +755,9 @@ int session_start(Session *s, sd_bus_message *properties, sd_bus_error *error) {
         log_struct(s->class == SESSION_BACKGROUND ? LOG_DEBUG : LOG_INFO,
                    "MESSAGE_ID=" SD_MESSAGE_SESSION_START_STR,
                    "SESSION_ID=%s", s->id,
-                   "USER_ID=%s", s->user->name,
+                   "USER_ID=%s", s->user->user_record->user_name,
                    "LEADER="PID_FMT, s->leader,
-                   LOG_MESSAGE("New session %s of user %s.", s->id, s->user->name));
+                   LOG_MESSAGE("New session %s of user %s.", s->id, s->user->user_record->user_name));
 
         if (!dual_timestamp_is_set(&s->timestamp))
                 dual_timestamp_get(&s->timestamp);
@@ -809,7 +808,10 @@ static int session_stop_scope(Session *s, bool force) {
         s->scope_job = mfree(s->scope_job);
 
         /* Optionally, let's kill everything that's left now. */
-        if (force || manager_shall_kill(s->manager, s->user->name)) {
+        if (force ||
+            (s->user->user_record->kill_processes != 0 &&
+             (s->user->user_record->kill_processes > 0 ||
+              manager_shall_kill(s->manager, s->user->user_record->user_name)))) {
 
                 r = manager_stop_unit(s->manager, s->scope, &error, &s->scope_job);
                 if (r < 0) {
@@ -825,14 +827,14 @@ static int session_stop_scope(Session *s, bool force) {
                  * Session stop is quite significant on its own, let's log it. */
                 log_struct(s->class == SESSION_BACKGROUND ? LOG_DEBUG : LOG_INFO,
                            "SESSION_ID=%s", s->id,
-                           "USER_ID=%s", s->user->name,
+                           "USER_ID=%s", s->user->user_record->user_name,
                            "LEADER="PID_FMT, s->leader,
                            LOG_MESSAGE("Session %s logged out. Waiting for processes to exit.", s->id));
         }
 
         return 0;
 }
-#else
+#else // 0
 static int session_stop_cgroup(Session *s, bool force) {
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
         int r;
@@ -840,7 +842,7 @@ static int session_stop_cgroup(Session *s, bool force) {
         assert(s);
 
         // elogind must not kill lingering user processes alive
-        if ( (force || manager_shall_kill(s->manager, s->user->name) )
+        if ( (force || manager_shall_kill(s->manager, s->user->user_record->user_name) )
             && (user_check_linger_file(s->user) < 1) ) {
                 r = session_kill(s, KILL_ALL, SIGTERM);
                 if (r < 0)
@@ -880,7 +882,7 @@ int session_stop(Session *s, bool force) {
         /* Kill cgroup */
 #if 0 /// elogind does not start scopes, but sessions
         r = session_stop_scope(s, force);
-#else
+#else // 0
         r = session_stop_cgroup(s, force);
         session_add_to_gc_queue(s);
 #endif // 0
@@ -907,7 +909,7 @@ int session_finalize(Session *s) {
                 log_struct(s->class == SESSION_BACKGROUND ? LOG_DEBUG : LOG_INFO,
                            "MESSAGE_ID=" SD_MESSAGE_SESSION_STOP_STR,
                            "SESSION_ID=%s", s->id,
-                           "USER_ID=%s", s->user->name,
+                           "USER_ID=%s", s->user->user_record->user_name,
                            "LEADER="PID_FMT, s->leader,
                            LOG_MESSAGE("Removed session %s.", s->id));
                 log_debug_elogind("Session %s not started, finalizing...", s->id);
@@ -1022,63 +1024,57 @@ static int get_process_ctty_atime(pid_t pid, usec_t *atime) {
 }
 
 int session_get_idle_hint(Session *s, dual_timestamp *t) {
-        usec_t atime = 0, n;
+        usec_t atime = 0;
         int r;
 
         assert(s);
 
-        /* Explicit idle hint is set */
-        if (s->idle_hint) {
+        /* Graphical sessions have an explicit idle hint */
+        if (SESSION_TYPE_IS_GRAPHICAL(s->type)) {
                 if (t)
                         *t = s->idle_hint_timestamp;
 
                 return s->idle_hint;
         }
 
-        /* Graphical sessions should really implement a real
-         * idle hint logic */
-        if (SESSION_TYPE_IS_GRAPHICAL(s->type))
-                goto dont_know;
-
-        /* For sessions with an explicitly configured tty, let's check
-         * its atime */
+        /* For sessions with an explicitly configured tty, let's check its atime */
         if (s->tty) {
                 r = get_tty_atime(s->tty, &atime);
                 if (r >= 0)
                         goto found_atime;
         }
 
-        /* For sessions with a leader but no explicitly configured
-         * tty, let's check the controlling tty of the leader */
+        /* For sessions with a leader but no explicitly configured tty, let's check the controlling tty of
+         * the leader */
         if (pid_is_valid(s->leader)) {
                 r = get_process_ctty_atime(s->leader, &atime);
                 if (r >= 0)
                         goto found_atime;
         }
 
-dont_know:
         if (t)
-                *t = s->idle_hint_timestamp;
+                *t = DUAL_TIMESTAMP_NULL;
 
-        return 0;
+        return false;
 
 found_atime:
         if (t)
                 dual_timestamp_from_realtime(t, atime);
 
-        n = now(CLOCK_REALTIME);
-
         if (s->manager->idle_action_usec <= 0)
-                return 0;
+                return false;
 
-        return atime + s->manager->idle_action_usec <= n;
+        return usec_add(atime, s->manager->idle_action_usec) <= now(CLOCK_REALTIME);
 }
 
-void session_set_idle_hint(Session *s, bool b) {
+int session_set_idle_hint(Session *s, bool b) {
         assert(s);
 
+        if (!SESSION_TYPE_IS_GRAPHICAL(s->type))
+                return -ENOTTY;
+
         if (s->idle_hint == b)
-                return;
+                return 0;
 
         s->idle_hint = b;
         dual_timestamp_get(&s->idle_hint_timestamp);
@@ -1090,6 +1086,8 @@ void session_set_idle_hint(Session *s, bool b) {
 
         user_send_changed(s->user, "IdleHint", "IdleSinceHint", "IdleSinceHintMonotonic", NULL);
         manager_send_changed(s->manager, "IdleHint", "IdleSinceHint", "IdleSinceHintMonotonic", NULL);
+
+        return 1;
 }
 
 int session_get_locked_hint(Session *s) {
@@ -1249,7 +1247,7 @@ bool session_may_gc(Session *s, bool drop_not_started) {
         }
 
         return true;
-#else
+#else // 0
         // elogind has to rely on drop_not_started, and that the state is correctly loaded
         return (drop_not_started || s->stopping);
 #endif // 0
@@ -1274,7 +1272,7 @@ SessionState session_get_state(Session *s) {
 
 #if 0 /// elogind does not support systemd scope_jobs
         if (s->scope_job || s->fifo_fd < 0)
-#else
+#else // 0
         if (s->fifo_fd < 0)
 #endif // 0
                 return SESSION_OPENING;
@@ -1293,7 +1291,7 @@ int session_kill(Session *s, KillWho who, int signo) {
                 return -ESRCH;
 
         return manager_kill_unit(s->manager, s->scope, who, signo, NULL);
-#else
+#else // 0
         if (who == KILL_LEADER) {
                 if (s->leader <= 0)
                         return -ESRCH;
@@ -1328,7 +1326,7 @@ static int session_open_vt(Session *s) {
         return s->vtfd;
 }
 
-int session_prepare_vt(Session *s) {
+static int session_prepare_vt(Session *s) {
         int vt, r;
         struct vt_mode mode = {};
 
@@ -1339,7 +1337,7 @@ int session_prepare_vt(Session *s) {
         if (vt < 0)
                 return vt;
 
-        r = fchown(vt, s->user->uid, -1);
+        r = fchown(vt, s->user->user_record->uid, -1);
         if (r < 0) {
                 r = log_error_errno(errno,
                                     "Cannot change owner of /dev/tty%u: %m",
@@ -1385,23 +1383,27 @@ error:
 }
 
 static void session_restore_vt(Session *s) {
-        int r, vt, old_fd;
+        int r;
 
-        /* We need to get a fresh handle to the virtual terminal,
-         * since the old file-descriptor is potentially in a hung-up
-         * state after the controlling process exited; we do a
-         * little dance to avoid having the terminal be available
-         * for reuse before we've cleaned it up.
-         */
-        old_fd = TAKE_FD(s->vtfd);
+        r = vt_restore(s->vtfd);
+        if (r == -EIO) {
+                int vt, old_fd;
 
-        vt = session_open_vt(s);
-        safe_close(old_fd);
+                /* It might happen if the controlling process exited before or while we were
+                 * restoring the VT as it would leave the old file-descriptor in a hung-up
+                 * state. In this case let's retry with a fresh handle to the virtual terminal. */
 
-        if (vt < 0)
-                return;
+                /* We do a little dance to avoid having the terminal be available
+                 * for reuse before we've cleaned it up. */
+                old_fd = TAKE_FD(s->vtfd);
+
+                vt = session_open_vt(s);
+                safe_close(old_fd);
+
+                if (vt >= 0)
+                        r = vt_restore(vt);
+        }
 
-        r = vt_restore(vt);
         if (r < 0)
                 log_warning_errno(r, "Failed to restore VT, ignoring: %m");
 
