@@ -10,7 +10,7 @@
 
 #include "alloc-util.h"
 #include "bus-error.h"
-#include "bus-util.h"
+#include "bus-polkit.h"
 #include "cgroup-util.h"
 #include "def.h"
 #include "device-util.h"
@@ -70,7 +70,7 @@ static int manager_new(Manager **ret) {
         m->session_units = hashmap_new(&string_hash_ops);
 
         if (!m->devices || !m->seats || !m->sessions || !m->sessions_by_leader || !m->users || !m->inhibitors || !m->buttons || !m->user_units || !m->session_units)
-#else
+#else // 0
         if (!m->devices || !m->seats || !m->sessions || !m->sessions_by_leader || !m->users || !m->inhibitors || !m->buttons)
 #endif // 0
                 return -ENOMEM;
@@ -113,7 +113,7 @@ static Manager* manager_unref(Manager *m) {
         if (!m)
                 return NULL;
 
-        log_debug_elogind("Tearing down all references (manager_unref) ...");
+        log_debug_elogind("%s", "Tearing down all references (manager_unref) ...");
         while ((session = hashmap_first(m->sessions)))
                 session_free(session);
 
@@ -756,7 +756,7 @@ static int manager_connect_bus(Manager *m) {
 
 #if 0 /// elogind has to setup its release agent
         return 0;
-#else
+#else // 0
         r = elogind_setup_cgroups_agent(m);
 
         return r;
@@ -1284,7 +1284,7 @@ static int run(int argc, char *argv[]) {
         (void) mkdir_label("/run/systemd/seats", 0755);
         (void) mkdir_label("/run/systemd/users", 0755);
         (void) mkdir_label("/run/systemd/sessions", 0755);
-#else
+#else // 0
         r = mkdir_label("/run/systemd", 0755);
         if ( (r < 0) && (-EEXIST != r) )
                 return log_error_errno(r, "Failed to create /run/systemd : %m");
@@ -1305,7 +1305,7 @@ static int run(int argc, char *argv[]) {
 #if 0 /// elogind also blocks SIGQUIT, and installs a signal handler for it
 
         assert_se(sigprocmask_many(SIG_BLOCK, NULL, SIGHUP, SIGTERM, SIGINT, SIGCHLD, -1) >= 0);
-#else
+#else // 0
         assert_se(sigprocmask_many(SIG_BLOCK, NULL, SIGHUP, SIGTERM, SIGINT, SIGQUIT, -1) >= 0);
 #endif // 0
 
