@@ -1,10 +1,13 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
+#if 0 /// UNNEEDED by elogind (only needed for testing __cpu_mask)
+#include <sched.h>
+#endif // 0
 #include <stdio.h>
 #include <string.h>
 
 #define __STDC_WANT_IEC_60559_TYPES_EXT__
-//#include <float.h>
+#include <float.h>
 
 #include "time-util.h"
 
@@ -69,6 +72,10 @@ int main(void) {
         info(uid_t);
         info(gid_t);
 
+#if 0 /// UNNEEDED by elogind (Only needed for *cpu_set* stuff in systemd we don't use)
+        info(__cpu_mask);
+#endif // 0
+
         info(enum Enum);
         info(enum BigEnum);
         info(enum BigEnum2);
